package fxPuntti;

import java.io.PrintStream;
import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.TextAreaOutputStream;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextArea;
import javafx.scene.text.Font;
import puntti.Liike;
import puntti.Puntti;
import puntti.SailoException;
import puntti.Tulos;


/**
 * Controlleri kutsuu luokkia ja siirtää ne käyttöliittymään
 * Kommunikoi käyttöliittymän ja koodin välillä
 * @author laspa
 * @version 20.1.2025
 *
 */
public class PunttiGUIController implements Initializable{
    
    @FXML private ListChooser<Liike> chooserLiikkeet;
    @FXML private ScrollPane panelLiike;


    
    @Override
    public void initialize(URL url, ResourceBundle bundle) {
        alusta();
    }
    
    @FXML void handleDefaultTallenna() {
        //Dialogs.showMessageDialog("Ei osata vielä tallentaa");
        tallenna();
    }
    

    @FXML void handleDefaultApua() {
        Dialogs.showMessageDialog("Ei osata vielä");
    }

    @FXML void handleDefaultMuokkaa() {
        Dialogs.showMessageDialog("Ei osata vielä");
    }

    @FXML void handleDefaultPoistaTulos() {
        Dialogs.showMessageDialog("Ei osata poistaa tulosta");
    }

    @FXML void handleDefaultTiedosto() {
        Dialogs.showMessageDialog("Ei osata vielä");
    }

    @FXML void handleDefaultUusiLiike() {
        //ModalController.showModal(PunttiGUIController.class.getResource("LisaaUusiLiikeGUIView.fxml"), "Liike", null, "");
        uusiLiike();
    }

    @FXML private void handleUusiTulos() {
        //ModalController.showModal(PunttiGUIController.class.getResource("LisaaUusiTulosGUIView.fxml"), "Tulos", null, "");
        uusiTulos();
    }

    //___________________________________________________________________ ALLE JAVA
    
    
    
    
    /**
     * puntti mitä käytetään
     */
    private Puntti puntti;
    private String kayttajanimi = "batman"; // ei vielä käytössä
    private TextArea areaLiike = new TextArea(); //Poista lopuksi
    
    
    
    /**
     * Tekee tarvittavat muut alustukset
     * vielä yksi teksikenttä  johon tulostetaan liikkeet ja tulokset
     * alustetaan myös liike listan kuuntelija
     */
    private void alusta() {
        chooserLiikkeet.clear();
        chooserLiikkeet.addSelectionListener(e -> naytaLiike());
        panelLiike.setContent(areaLiike);
        areaLiike.setFont(new Font("Arial", 12));
        panelLiike.setFitToHeight(true);
        
    }
    
    
    /**
     * näyttää valitun liikkeen tiedot tekstiruudussa
     */
    private void naytaLiike() {
        Liike liikeKohdalla = chooserLiikkeet.getSelectedObject();
        
        if (liikeKohdalla == null) return;
        
        areaLiike.setText("");
        try(PrintStream os = TextAreaOutputStream.getTextPrintStream(areaLiike)){
                os.println("_____________________________________");
                liikeKohdalla.tulosta(os);
                os.println("_____________________________________");
                List<Tulos> tulokset = puntti.annaTulokset(liikeKohdalla);
                for (Tulos tulo: tulokset)
                    tulo.tulosta(os);
                os.println("_____________________________________");
        }
    }
    
    
    


    
    
    /**
     * Alustaa puntin lukemalla käyttäjänimeä vastaavan tiedoston
     * @param nimi tiedosto josta käyttäjän tiedot luetaan
     */
    protected void lueTiedosto(String nimi) {
        kayttajanimi = nimi;
        try{
            puntti.lueTiedostosta(kayttajanimi);
            hae(0);
        }catch (SailoException e) {
            Dialogs.showMessageDialog(e.getMessage());
        }
    }
    
    /**
     * tallentaa tiedot
     */
    private void tallenna(){
        try {
            puntti.tallenna();
        }catch (SailoException e) {
            Dialogs.showMessageDialog(e.getMessage());
        }
    }
    
    
    /**
     * @return palauttaa true jos avataan ja false jos ei avata
     */
    public boolean avaa() {
        String uusinimi = KirjautumissivuGUIController.kysyNimi(null, "batman");
        if (uusinimi == null) return false;
        lueTiedosto(uusinimi);
        return true;
    }
      


    /**
     * Asetetaan käytettävä puntti
     * @param puntti jota käytetään
     */
    public void setPuntti(Puntti puntti) {
        this.puntti = puntti;
        
    }
    
    
    
    /**
     * Hakee liikkeen tiedot lstaan
     * @parm lid liike id joka on voimassa haun jälkeen
     */
    private void hae(int lid) {
        chooserLiikkeet.clear();
        
        int index = 0;
        for (int i=0; i < puntti.getLiikkeita(); i++) {
            Liike liike = puntti.annaLiike(i);
            if(liike.getLid() == lid) index = i;
            chooserLiikkeet.add(""+liike.getNimi(), liike);
        }
        chooserLiikkeet.setSelectedIndex(index);
        
    }
    
    
    /**
     * Lisätään Punttiin uusi liike
     */
    private void uusiLiike() {
        Liike uusi = new Liike();
        uusi.lisaaLiike();
        uusi.taytaKyykky();
        try {
            puntti.lisaa(uusi);
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ounou ongelma lisätessä liikettä" + e.getMessage());
        }
        hae(uusi.getLid());
    }
    
    
    /**
     * Lisätään Punttiin uusi tulos
     */
    private void uusiTulos() {
        Liike liikeKohdalla = chooserLiikkeet.getSelectedObject();
        if (liikeKohdalla == null) return;
        Tulos tulo = new Tulos();
        tulo.lisaaTulos();
        tulo.taytaPenkkiTulos(liikeKohdalla.getLid()); 
        puntti.lisaa(tulo);
        hae(liikeKohdalla.getLid());
        
    }
    
   

}