/**
 * 
 */
package puntti;

import java.io.File;
import java.util.List;

/**
 * |------------------------------------------------------------------------|
 * | Luokan nimi:  Puntti                               | Avustajat:        |
 * |-------------------------------------------------------------------------
 * | Vastuualueet:                                      |                   |
 * |                                                    | - Liikkeet        |
 * | - Huolehtii Liikkeet ja Tulokset -luokkien         | - Tulos           |
 * | välisestä yhteistyöstä ja välittää näitä tietoja   | - Liike           |
 * | pyydettäessä                                       | - Tulokset        |
 * | - lukee ja kirjoittaa kerhon tiedostoon pyytämällä |                   |
 * | apuja avustajiltaan                                |                   |
 * |-------------------------------------------------------------------------
 * 
 * @author laspa
 * @version 5.3.2025
 *
 */
public class Puntti {
    
    private Liikkeet liikkeet = new Liikkeet();
    private Tulokset tulokset = new Tulokset();
    
    private String hakemisto = "batman";
    
    
    /**
     * Palautaa punti nliikkeiden määrän
     * @return liikemäärän
     */
    public int getLiikkeita() {
        return liikkeet.getLkm();
    }

    
     /**
     * Lisätään uusi liike
     * @param liike lisättävä liike
     * @throws SailoException kun lisääminen ei onnistu
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Puntti puntti = new Puntti();
     * Liike kyykky = new Liike(), kyykky2 = new Liike();
     * kyykky.lisaaLiike(); kyykky2.lisaaLiike();
     * puntti.getLiikkeita() === 0;
     * puntti.lisaa(kyykky); puntti.getLiikkeita() === 1;
     * puntti.lisaa(kyykky2); puntti.getLiikkeita() === 2;
     * puntti.lisaa(kyykky); puntti.getLiikkeita() === 3;
     * puntti.getLiikkeita() === 3;
     * puntti.annaLiike(0) === kyykky;
     * puntti.annaLiike(1) === kyykky2;
     * puntti.annaLiike(2) === kyykky;
     * puntti.annaLiike(3) === kyykky; #THROWS IndexOutOfBoundsException 
     * puntti.lisaa(kyykky); puntti.getLiikkeita() === 4;
     * puntti.lisaa(kyykky); puntti.getLiikkeita() === 5;
     * puntti.lisaa(kyykky);            #THROWS SailoException
     * </pre>
     */
    public void lisaa(Liike liike) throws SailoException {
        liikkeet.lisaa(liike);
    }
    
    
    /**
     * Lisää uuden tuloksen punttiin
     * @param tulos lisättävä tulos
     */
    public void lisaa(Tulos tulos) {
        tulokset.lisaa(tulos);
    }
    
    
    /**
     * Palauttaa i:n liikkeen
     * @param i mones liike palautetaan
     * @return viite i:teen liikkeeseen
     * @throws IndexOutOfBoundsException jos i on epäsopiva
     */
    public Liike annaLiike(int i) {
        return liikkeet.anna(i);
    }
    
    
    /**
     * Haetaan liikkeen tulokset
     * @param liike jolle tuloksia haetaan
     * @return tietorakenne missä viitteet liikkeen tuloksiin
     * @example
     * <pre name="test">
     * #import java.util.*;
     * 
     * Puntti puntti = new Puntti();
     * Liike kyykky = new Liike(), kyykky2 = new Liike(), kyykky3 = new Liike();
     * kyykky.lisaaLiike(); kyykky2.lisaaLiike(); kyykky3.lisaaLiike();
     *  int lid = kyykky.getLid();
     *  int lid2 = kyykky2.getLid();
     *  Tulos penkki1 = new Tulos(lid); puntti.lisaa(penkki1);
     *  Tulos penkki2 = new Tulos(lid); puntti.lisaa(penkki2);
     *  Tulos penkki3 = new Tulos(lid2); puntti.lisaa(penkki3);
     *  Tulos penkki4 = new Tulos(lid2); puntti.lisaa(penkki4);
     *  Tulos penkki5 = new Tulos(lid2); puntti.lisaa(penkki5);
     *  
     *  List<Tulos> loytyneet;
     *  loytyneet = puntti.annaTulokset(kyykky3);
     *  loytyneet.size() === 0; 
     *  loytyneet = puntti.annaTulokset(kyykky);
     *  loytyneet.size() === 2; 
     *  loytyneet.get(0) == penkki1 === true;
     *  loytyneet.get(1) == penkki2 === true;
     *  loytyneet = puntti.annaTulokset(kyykky2);
     *  loytyneet.size() === 3; 
     *  loytyneet.get(0) == penkki3 === true;
     * </pre> 
     */

    public List<Tulos> annaTulokset(Liike liike) {
        return tulokset.annaTulokset(liike.getLid());
    }
    
    
    /**
     * Lukee käyttäjän tiedot tiedostoista
     * @param nimi käyttäjä nimi
     * @throws SailoException jos tiedoston lukeminen ei onnistu
     */
    public void lueTiedostosta(String nimi) throws SailoException {
        File hake = new File(nimi);
        hake.mkdir();
        liikkeet = new Liikkeet();
        tulokset = new Tulokset();

        hakemisto = nimi;
        liikkeet.lueTiedostosta(nimi);
        tulokset.lueTiedostosta(nimi);
    }

    
    
    /**
     * tallentaa käyttäjän tiedot tiedostoon
     * @throws SailoException jos tallennus ei onnistu
     */
    public void tallenna() throws SailoException {
        String virhe = "";
        try {
            liikkeet.tallenna(hakemisto);
        } catch ( SailoException ex ) {
            virhe = ex.getMessage();
        }

        try {
            tulokset.tallenna(hakemisto);
        } catch ( SailoException ex ) {
            virhe += ex.getMessage();
        }
        if ( !"".equals(virhe) ) throw new SailoException(virhe);

    }
    

    /**
     * Testiohjelma Puntti.javalle
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Puntti puntti = new Puntti();
        
        
        
        try {
            
            Liike kyykky = new Liike();
            Liike kyykky2 = new Liike();
            
            kyykky.lisaaLiike();
            kyykky.taytaKyykky();
            
            kyykky2.lisaaLiike();
            kyykky2.taytaKyykky();
            
            puntti.lisaa(kyykky);
            puntti.lisaa(kyykky2);
            
            
            Tulos penkki1 = new Tulos();
            penkki1.taytaPenkkiTulos(kyykky2.getLid());
            Tulos penkki2 = new Tulos();
            penkki2.taytaPenkkiTulos(kyykky.getLid());
            Tulos penkki3 = new Tulos();
            penkki3.taytaPenkkiTulos(kyykky2.getLid());
            Tulos penkki4 = new Tulos();
            penkki4.taytaPenkkiTulos(kyykky2.getLid());
            
            puntti.lisaa(penkki1);
            puntti.lisaa(penkki2);
            puntti.lisaa(penkki3);
            puntti.lisaa(penkki4);
            
            for (int i = 0; i < puntti.getLiikkeita(); i++) {
                Liike liike = puntti.annaLiike(i);
                System.out.println("Liike indeksi " + i);
                liike.tulosta(System.out);
            
            
            List<Tulos> tulokset = puntti.annaTulokset(liike);
                    
                    for (Tulos tulo: tulokset) {
                        System.out.println(tulo.getLid() + " ");
                    tulo.tulosta(System.out);
                }
                    System.out.println();
            
            }
            } catch (SailoException e) {
            System.err.println(e.getMessage());
            
        }
        
        
    


}


}
